// time.c
// 

#include <pktlab.h>

#include <stdint.h>
#include <time.h>

//
// INTERNAL MACROS
//

//
// INTERNAL FUNCTION DECLARATIONS
//

static int is_leap_year(int year);

//
// EXPORTED FUNCTION DEFINITIONS
//

pktlab_time_t pktlab_time_now(void) {
	struct timeval tv;
	
	gettimeofday(&tv, NULL);
	return pktlab_timeval_to_time(&tv);
}

pktlab_time_t pktlab_tm_to_time(const struct tm * tm) {
	const uint_fast32_t sec_per_min  = 60;
	const uint_fast32_t sec_per_hour = 3600;
	const uint_fast32_t sec_per_day = 86400;
	const uint_fast32_t day_per_common_year = 365;
	uint_fast32_t sec_from_epoch;
	int i;

	sec_from_epoch = 0;
	for (i = 1970; i < 1900+tm->tm_year; ++i)
		sec_from_epoch += ((is_leap_year(i)) ?
			(day_per_common_year+1):(day_per_common_year)) * sec_per_day;

	sec_from_epoch += tm->tm_yday * sec_per_day;
	sec_from_epoch += tm->tm_hour * sec_per_hour;
	sec_from_epoch += tm->tm_min * sec_per_min;
	sec_from_epoch += tm->tm_sec;

	return pktlab_time_sec(sec_from_epoch);
}

//
// INTERNAL FUNCTION DEFINITIONS
//

int is_leap_year(int year) {
	if (year % 4 != 0) {
		return 0;
	} else if (year % 100 != 0) {
		return 1;
	} else if (year % 400 != 0) {
		return 0;
	}
	return 1;
}