// main.c
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>

#include <pktlab.h>

#include "manager.h"

#define USAGE "pktlabme [-h] [-c ME_CONF_PATH] [ADDR[:PORT]]"
#define MECONF_PATH "me.conf" // Should be configured with autoconf

//
// TYPE DEFINITIONS
//

// ...

//
// RUN-TIME PARAMETERS
//
// TODO: parse more arguments: broker address, certificates file, xsub channel
// list
struct {
    const char *ctlspec;
    const char *confpath;
    const char *brokerspec;
    const char *const *certpaths;
    const char *troot;
} _args;

//
// GLOBAL VARIABLES
//

// ...

//
// INTERNAL FUNCTION DECLARATIONS
//

static void initialize(int argc, char *const argv[]);

//
// MAIN FUNCTION DEFINITION
//

int main(int argc, char *const argv[]) {
    initialize(argc, argv);

    manager_init();
    manager_start(_args.confpath, _args.ctlspec);
    manager_wait();

    exit(EXIT_SUCCESS);
    return 0;
}

//
// INTERNAL FUNCTION DEFINITIONS
//

void initialize(int argc, char *const argv[]) {
    static const char *const default_ctlspec = PKTLAB_DEFAULT_SPEC_STR;
    struct sigaction action;
    int optch;

    // Disable SIGPIPE signal; handle broken pipe error on write.

    memset(&action, 0, sizeof(action));
    action.sa_handler = SIG_IGN;
    sigaction(SIGPIPE, &action, NULL);

    // Close stdin and stdout.

    fclose(stdin);
    fclose(stdout);

    // Parse command-line arguments.

    while ((optch = getopt(argc, argv, "c:h")) != -1) {
        switch (optch) {
        case 'c':
            _args.confpath = optarg;
            break;
        case 'h':
        default:
            goto usage_error;
        }
    }

    argc -= optind;
    argv += optind;

    // set up default option/argument values
    if (argc > 0)
        _args.ctlspec = *argv;
    else
        _args.ctlspec = default_ctlspec;

    if (_args.confpath == NULL)
        _args.confpath = MECONF_PATH;

    return;

usage_error:
    fprintf(stderr, "USAGE: " USAGE "\n");
    exit(EXIT_FAILURE);
    return;
}
