// raw_dummy.c
// pktlab dummy raw socket impl. for non-supporting OS
//

#include <stdint.h>
#include <sys/types.h>

#include <pktlab.h>

#include "console.h"
#include "raw.h"

//
// EXPORTED FUNCTION DEFINITIONS
//

struct raw_socket_obj * create_raw_socket_obj(void) {
    trace("%s()", __func__);
    debug("%s not supported due to OS restriction");
    return NULL;
}

int raw_socket_setup_v4 (
    const uint8_t * intfaddr,
    struct raw_prog * cert_prog_ll,
    struct raw_socket_obj * obj)
{
    trace("%s(intfaddr:%p,cert_prog_ll:%p)",
        __func__, intfaddr, cert_prog_ll);
    debug("%s not supported due to OS restriction");
    return RAW_UNKFAULT;
}

int raw_socket_setup_v6 (
    const uint8_t * intfaddr,
    struct raw_prog * cert_prog_ll,
    struct raw_socket_obj * obj)
{
    trace("%s(intfaddr:%p,cert_prog_ll:%p)",
        __func__, intfaddr, cert_prog_ll);
    debug("%s not supported due to OS restriction");
    return RAW_UNKFAULT;
}

void raw_socket_close(struct raw_socket_obj * obj) {
    trace("%s(...)", __func__);
    debug("%s not supported due to OS restriction");
    return;
}

ssize_t raw_socket_send (
    const void * buf, size_t len, int flags,
    struct raw_socket_obj * obj, enum raw_error * err)
{
    trace("%s(len:%zu,flags:%d)",
        __func__, len, flags);
    debug("%s not supported due to OS restriction");
    if (err != NULL)
        *err = RAW_UNKFAULT;
    return -1;
}

ssize_t raw_socket_recv (
    void * buf, size_t len, int flags,
    struct raw_socket_obj * obj, enum raw_error * err)
{
    trace("%s(len:%zu,flags:%d)",
        __func__, len, flags);
    debug("%s not supported due to OS restriction");
    if (err != NULL)
        *err = RAW_UNKFAULT;
    return -1;
}

int raw_socket_set_recvfilter (
    struct raw_prog * filter,
    struct raw_socket_obj * obj)
{
    trace("%s(filter:%p)", __func__, filter);
    debug("%s not supported due to OS restriction");
    return RAW_UNKFAULT;
}

void raw_v4_hdrinfo_read ( // do nothing
    const struct pktlab_vmem_region * restrict rgn,
    uint_fast32_t off, uint_fast32_t len, void * buf)
{
    trace("%s(off:%u,len:%u)", __func__,
        (unsigned) off, (unsigned) len);
    debug("%s not supported due to OS restriction");
}

void raw_v6_hdrinfo_read ( // do nothing
    const struct pktlab_vmem_region * restrict rgn,
    uint_fast32_t off, uint_fast32_t len, void * buf)
{
    trace("%s(off:%u,len:%u)", __func__,
        (unsigned) off, (unsigned) len);
    debug("%s not supported due to OS restriction");
}