#include <string.h>
#include "tls_util.h"
struct pktlab_privatekey *get_pem_privkey(const char *pem) {
    struct pktlab_privatekey *k;

    k = pktlab_create_privatekey();
    if (pktlab_load_privatekey_pem(pem, strlen(pem), NULL, k) < 0) {
        pktlab_cleanup_privatekey(k);
        return NULL;
    }

    return k;
}

struct pktlab_certificate *get_pem_cert(const char *pem) {
    struct pktlab_certificate *cert;

    cert = pktlab_create_cert();
    if (pktlab_load_cert_pem(pem, strlen(pem), cert) < 0) {
        pktlab_cleanup_cert(cert);
        cert = NULL;
    }

    return cert;
}

int get_cert_ls(const char **certpem_ls, size_t certnum,
                struct pktlab_certificate **cert_ls) {
    size_t i;
    struct pktlab_certificate *cert;

    for (i = 0; i < certnum; ++i) {
        cert = get_pem_cert(certpem_ls[i]);
        if (cert == NULL) {
            certnum = i;
            goto fail_cert;
        }

        cert_ls[i] = cert;
    }

    return 0;

fail_cert:
    free_cert_ls(cert_ls, certnum);
    return -1;
}

void free_cert_ls(struct pktlab_certificate **cert_ls, uint_fast32_t cert_num) {
    struct pktlab_certificate **cert;
    uint_fast32_t i;

    if (cert_ls == NULL || cert_num == 0)
        return;

    cert = cert_ls;
    for (i = 0; i < cert_num; ++i) {
        pktlab_cleanup_cert(*cert);
        ++cert;
    }

    return;
}

void free_pubkey_ls(struct pktlab_publickey **pubkey_ls,
                    uint_fast32_t key_num) {
    struct pktlab_publickey **pubkey;
    uint_fast32_t i;

    if (pubkey_ls == NULL || key_num == 0)
        return;

    pubkey = pubkey_ls;
    for (i = 0; i < key_num; ++i) {
        pktlab_cleanup_publickey(*pubkey);
        ++pubkey;
    }

    return;
}

int get_pubkey_ls(const char **pubkeypem_ls, size_t pubkeynum,
                  struct pktlab_publickey **pubkey_ls) {
    size_t i;
    struct pktlab_publickey *pubkey;

    for (i = 0; i < pubkeynum; ++i) {
        pubkey = get_pem_pubkey(pubkeypem_ls[i]);
        if (pubkey == NULL) {
            pubkeynum = i;
            goto fail_pubkey;
        }

        pubkey_ls[i] = pubkey;
    }

    return 0;

fail_pubkey:
    free_pubkey_ls(pubkey_ls, pubkeynum);
    return -1;
}

struct pktlab_publickey *get_pem_pubkey(const char *pem) {
    struct pktlab_publickey *k;

    k = pktlab_create_publickey();
    if (pktlab_load_publickey_pem(pem, strlen(pem), k) < 0) {
        pktlab_cleanup_publickey(k);
        return NULL;
    }

    return k;
}
